export declare const unknownValue = -1n;
/**
 * Object for memory entries
 *
 * @member {string} varName Variable name defined in assembly
 * @member {bigint} value Variable value (64-bit unsigned)
 * @member {string} shadow If content is the same as other variable,
 * this will have the other variable name.
 */
export interface MemoryObj {
    varName: string;
    value: bigint;
    shadow: string;
}
export declare const Constants: {
    minus1: bigint;
    pow2to64: bigint;
    maxPositive: bigint;
    numberMaxPositive: number;
};
export declare const utils: {
    unsigned2signed(unsigned: bigint): bigint;
    signed2unsigned(signed: bigint): bigint;
};
export declare class CONTRACT {
    Memory: MemoryObj[];
    asmCodeArr: string[];
    constructor(asmCode: string[]);
    unknownMemory(keepRegisters: boolean, keepSuperRegisters: boolean): void;
    unknownSuperRegisterA(): void;
    unknownSuperRegisterB(): void;
    getMemoryByName(name: string): MemoryObj;
    revokeShadow(changedVar: string): void;
    unknownAndRevoke(Var: MemoryObj): void;
    setAndRevoke(AssignedVar: MemoryObj, Variable: MemoryObj): void;
    zeroAndRevoke(Var: MemoryObj): void;
    /**
     * Runs only one instruction (step into)
     *
     * @return string indicating error/status. Empty string on success.
     */
    optimize(): string[];
}
