import { CPU } from './cpu';
export const unknownValue = -1n;
export const Constants = {
    // do not change these
    minus1: 18446744073709551615n,
    pow2to64: 18446744073709551616n,
    maxPositive: 9223372036854775807n,
    numberMaxPositive: 9223372036854775000
};
export const utils = {
    unsigned2signed(unsigned) {
        unsigned %= 1n << 64n;
        if (unsigned >= (1n << 63n)) {
            return unsigned - (1n << 64n);
        }
        return unsigned;
    },
    signed2unsigned(signed) {
        signed %= 18446744073709551616n;
        if (signed < 0) {
            return signed + 18446744073709551616n;
        }
        return signed;
    }
};
export class CONTRACT {
    constructor(asmCode) {
        this.Memory = [];
        this.asmCodeArr = asmCode;
        CPU.cpuDeploy(this);
        this.Memory.push({ varName: 'A1', value: unknownValue, shadow: '' }, { varName: 'A2', value: unknownValue, shadow: '' }, { varName: 'A3', value: unknownValue, shadow: '' }, { varName: 'A4', value: unknownValue, shadow: '' }, { varName: 'B1', value: unknownValue, shadow: '' }, { varName: 'B2', value: unknownValue, shadow: '' }, { varName: 'B3', value: unknownValue, shadow: '' }, { varName: 'B4', value: unknownValue, shadow: '' });
    }
    unknownMemory(keepRegisters, keepSuperRegisters) {
        this.Memory.forEach((Mem) => {
            if (keepRegisters && /^r\d$/.test(Mem.varName)) {
                return;
            }
            if (keepSuperRegisters && /^[AB][1234]$/.test(Mem.varName)) {
                return;
            }
            if (/^n\d+$/.test(Mem.varName)) {
                Mem.value = BigInt(Mem.varName.substring(1));
                Mem.shadow = '';
                return;
            }
            Mem.value = unknownValue;
            Mem.shadow = '';
        });
    }
    unknownSuperRegisterA() {
        this.Memory.forEach((Mem) => {
            if (/^A[1234]$/.test(Mem.varName)) {
                Mem.value = unknownValue;
                Mem.shadow = '';
                return;
            }
            if (/^A[1234]$/.test(Mem.shadow)) {
                Mem.shadow = '';
            }
        });
    }
    unknownSuperRegisterB() {
        this.Memory.forEach((Mem) => {
            if (/^B[1234]$/.test(Mem.varName)) {
                Mem.value = unknownValue;
                Mem.shadow = '';
                return;
            }
            if (/^B[1234]$/.test(Mem.shadow)) {
                Mem.shadow = '';
            }
        });
    }
    getMemoryByName(name) {
        const RetObj = this.Memory.find(Mem => Mem.varName === name);
        if (RetObj === undefined) {
            throw new Error(`optimizerVM: getMemoryByName: Variable '${name}' not declared.`);
        }
        return RetObj;
    }
    revokeShadow(changedVar) {
        this.Memory.forEach((Mem) => {
            if (Mem.shadow === changedVar) {
                Mem.shadow = '';
            }
        });
    }
    unknownAndRevoke(Var) {
        Var.value = unknownValue;
        Var.shadow = '';
        this.revokeShadow(Var.varName);
    }
    setAndRevoke(AssignedVar, Variable) {
        AssignedVar.value = Variable.value;
        AssignedVar.shadow = Variable.varName;
        this.revokeShadow(AssignedVar.varName);
        Variable.shadow = AssignedVar.varName;
    }
    zeroAndRevoke(Var) {
        if (Var.value !== 0n) {
            Var.value = 0n;
            Var.shadow = '';
            this.revokeShadow(Var.varName);
        }
    }
    /**
     * Runs only one instruction (step into)
     *
     * @return string indicating error/status. Empty string on success.
     */
    optimize() {
        let cpuExitCode;
        this.asmCodeArr.forEach((code, line) => {
            cpuExitCode = CPU.cpu(this, line);
            if (cpuExitCode === null) {
                this.asmCodeArr[line] = 'DELETE';
            }
        });
        return this.asmCodeArr.filter(code => code !== 'DELETE');
    }
}
